
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#ifndef DEFINES_H
#define DEFINES_H

#define DEBUGGING              1

#ifdef _WIN32
    #define NO_OUTPUT_FILE        ":NUL"
#else
    #define NO_OUTPUT_FILE        "/dev/null"
#endif

#define TMP_YUV_FILE          "tmp.yuv"
#define NUM_BITS              10

#define NO_TEST_FILT           3       // Filter supports (5/7/9)
#define NO_PREV_FILTERS       16

#define NO_VAR_BINS           16
#define NO_FILTERS            16

#define FILTER_LENGTH          9
#define SQUARE(X)             ((X) * (X))
#define MAX_SQR_FILT_LENGTH   (SQUARE(FILTER_LENGTH / 2 + 1) + 1)
#define SQR_FILT_LENGTH_9SYM  (SQUARE(5) + 1)
#define SQR_FILT_LENGTH_7SYM  (SQUARE(4) + 1)
#define SQR_FILT_LENGTH_5SYM  (SQUARE(3) + 1)

#define imgpel unsigned short
#define int8 char
typedef unsigned char byte;    //!< byte type definition

#ifndef max
#define max(a, b)	(((a) > (b))? (a): (b))
#endif

#ifndef min
#define min(a, b)	(((a) < (b))? (a): (b))
#endif

#define ROUND(a)  (((a) < 0)? (int)((a) - 0.5) : (int)((a) + 0.5))

#define FATAL_ERROR_0(MESSAGE, EXITCODE)                                \
          {                                                             \
            printf(MESSAGE);                                            \
            exit(EXITCODE);                                             \
          }

#define FATAL_ERROR_1(MESSAGE, PARAM_1, EXITCODE)                       \
          {                                                             \
            printf(MESSAGE, PARAM_1);                                   \
            exit(EXITCODE);                                             \
          }

#define FATAL_ERROR_2(MESSAGE, PARAM_1, PARAM_2, EXITCODE)              \
          {                                                             \
            printf(MESSAGE, PARAM_1, PARAM_2);                          \
            exit(EXITCODE);                                             \
          }

typedef struct
{
  imgpel **imgY;
  imgpel ***imgUV;
  int img_height;
  int img_width;
} imgStruct;

#include "IO.h"

//
// results.c
//
typedef struct {
  FILE *dataFile;
  int noFr;  
  double snr_y;
  double snrAll;
  int bits;
  int bitAll;
} results;

void resultsInit(results *r);
void resultsEnd(results r);
void printResults(results *r, int no);
void printResultsFinal(results *r);
void updateBits(results *r, int bits);
void find_snr(results *r, imgpel **imgY_org, imgpel **imgY_rec, int height, int width);


//
//  memalloc.c
//
void initMatrix_int(int ***m2D, int d1, int d2);
void initMatrix_double(double ***m2D, int d1, int d2);
void initMatrix_ushort(unsigned short ***m2D, int d1, int d2);
void initMatrix3D_int(int ****m3D, int d1, int d2, int d3);
void initMatrix3D_double(double ****m3D, int d1, int d2, int d3);
void initMatrix3D_ushort(unsigned short ****m3D, int d1, int d2, int d3);
void initMatrix4D_int(int *****m4D, int d1, int d2, int d3, int d4);
void initMatrix4D_double(double *****m4D, int d1, int d2, int d3, int d4);
void initMatrix4D_ushort(unsigned short *****m4D, int d1, int d2, int d3, int d4);
void destroyMatrix_int(int **m2D);
void destroyMatrix_double(double **m2D);
void destroyMatrix_ushort(unsigned short **m2D);
void destroyMatrix3D_int(int ***m3D, int d1);
void destroyMatrix3D_double(double ***m3D, int d1);
void destroyMatrix3D_ushort(unsigned short ***m3D, int d1);
void destroyMatrix4D_int(int ****m4D, int d1, int d2);
void destroyMatrix4D_double(double ****m4D, int d1, int d2);
void destroyMatrix4D_ushort(unsigned short ****m4D, int d1, int d2);


// 
//  misc.c
//
int Clip3Fun(int low, int high, int val);
void int2Byte(imgStruct *imgIn, int width, int height);
void initImg(imgStruct *imgIn, int height, int width);
void padImage(imgpel **imgY,  imgpel **imgY_pad, int fl, int img_height, int img_width);
void calcVar(imgpel **imgY_var, imgpel **imgY_rec, int fl, int img_height, int img_width);


// 
//  decVarPostprocessing.c
//
double decFilterVarPred(imgpel **imgY_rec, imgpel **imgY_org, imgpel **varImg, int filtNo_best, double **filterCoeffSym, 
                       int varIndTab[NO_VAR_BINS], int usePrevFiltBest[NO_VAR_BINS], int prevFiltInd, int codedFlag,
                       int img_height, int img_width);

#endif // DEFINES_H

